@@ -1,5 +1,3 @@
-/* $XConsortium: x2jet.c,v 1.6 94/04/17 20:44:03 rws Exp $ */
-
 /* -*-C-*-
 ********************************************************************************
 *
@@ -660,7 +658,7 @@ void prepare_color_mapping (
     setup_RGBshiftmask(&color.sm, xwd_header.red_mask,
 		       xwd_header.green_mask, xwd_header.blue_mask);
   else {
-    if (!(colormap = (long *) malloc(xwd_header.ncolors * sizeof(long))))
+    if (!(colormap = malloc(xwd_header.ncolors * sizeof(long))))
       fatal_err((catgets(nlmsg_fd,NL_SETN,24,
 		"Could not allocate memory for X-to-printer colormap.")));
 
@@ -939,7 +937,7 @@ void add_index_to_chain (
    * if possible, otherwise malloc space.
    */
   if (color.freechain == NULL) {
-    if (!(new = (COLORINDEX *) malloc(sizeof(COLORINDEX))))
+    if (!(new = malloc(sizeof(COLORINDEX))))
       fatal_err((catgets(nlmsg_fd,NL_SETN,8,
 			"Could not allocate memory for color translation.")));
   } else {
@@ -1056,13 +1054,21 @@ void validate_visual(void)
 
   switch (xwd_header.visual_class) {
   case GrayScale:
-    if (depth > 8)  invalid_depth_for_visual(depth, "GrayScale");    break;
+    if (depth > 8)
+        invalid_depth_for_visual(depth, "GrayScale");
+    break;
   case StaticGray:
-    if (depth > 8)  invalid_depth_for_visual(depth, "StaticGray");   break;
+    if (depth > 8)
+        invalid_depth_for_visual(depth, "StaticGray");
+    break;
   case PseudoColor:
-    if (depth > 8)  invalid_depth_for_visual(depth, "PseudoColor");  break;
+    if (depth > 8)
+        invalid_depth_for_visual(depth, "PseudoColor");
+    break;
   case StaticColor:
-    if (depth > 8)  invalid_depth_for_visual(depth, "StaticColor");  break;
+    if (depth > 8)
+        invalid_depth_for_visual(depth, "StaticColor");
+    break;
   case DirectColor:
     if (depth != 12 && depth != 24)
 	invalid_depth_for_visual(depth, "DirectColor");
@@ -1110,7 +1116,7 @@ void read_xwd_data (FILE *in)
     }
 
     /* Allocate space for xwd color structures */
-    if (!(xwd_colors = (XColor*) malloc(sizeof(XColor) * xwd_header.ncolors)))
+    if (!(xwd_colors = malloc(sizeof(XColor) * xwd_header.ncolors)))
     	fatal_err((catgets(nlmsg_fd,NL_SETN,12,
 			"Could not allocate memory for xwdfile color table.")));
 
@@ -1279,7 +1285,10 @@ void write_image_suffix (
   if (device == PJETXL)
      fprintf(out,"\033*rC");
   else
+  {
      fprintf(out,"\033*rB");
+     fprintf(out,"\033E");
+  }
 
   /* If doing transparencies, tell it to stop */
   if (slide && device != LJET)
@@ -1520,7 +1529,7 @@ void write_portrait_Z_image (
   int height = limit.height;
   long *line, *lp;
 
-  if (!(line = (long *) malloc(width * sizeof(long))))
+  if (!(line = malloc(width * sizeof(long))))
     fatal_err((catgets(nlmsg_fd,NL_SETN,18,
 			"Could not allocate memory for image line buffer.")));
 
@@ -1545,7 +1554,7 @@ void write_landscape_Z_image (
   int height = limit.width;
   long *line, *lp;
 
-  if (!(line = (long *) malloc(width * sizeof(long))))
+  if (!(line = malloc(width * sizeof(long))))
     fatal_err((catgets(nlmsg_fd,NL_SETN,19,
 			"Could not allocate memory for image line buffer.")));
 
@@ -1570,7 +1579,7 @@ void write_portrait_XY_image (
   int height = limit.height;
   long *line, *lp;
 
-  if (!(line = (long *) malloc(width * sizeof(long))))
+  if (!(line = malloc(width * sizeof(long))))
     fatal_err((catgets(nlmsg_fd,NL_SETN,20,
 			"Could not allocate memory for image line buffer.")));
 
@@ -1595,7 +1604,7 @@ void write_landscape_XY_image (
   int height = limit.width;
   long *line, *lp;
 
-  if (!(line = (long *) malloc(width * sizeof(long))))
+  if (!(line = malloc(width * sizeof(long))))
     fatal_err((catgets(nlmsg_fd,NL_SETN,21,
 			"Could not allocate memory for image line buffer.")));
 
