@@ -33,12 +33,12 @@ from the X Consortium.
  *
  * Author: Michael R. Gretzinger, MIT Project Athena
  *
- * Modified by Marvin Solomon, Univeristy of Wisconsin, to handle Apple
+ * Modified by Marvin Solomon, University of Wisconsin, to handle Apple
  * Laserwriter (PostScript) devices (-device ps).
  * Also accepts the -compact flag that produces more compact output
  * by using run-length encoding on white (1) pixels.
  * This version does not (yet) support the following options
- *   -append -dump -noff -nosixopt -split
+ *   -append -dump -noff -split
  *
  * Changes
  * Copyright 1986 by Marvin Solomon and the University of Wisconsin
@@ -69,6 +69,29 @@ from the X Consortium.
  * to devices recognized.  Also added -density, -cutoff, and -noposition
  * command line options.
  *
+ * Modified by Jay Hobson, Sun Microsystems to support paper sizes other
+ * than 8.5x11 inches.
+ *
+ * Copyright (c) 2002, Oracle and/or its affiliates.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -84,7 +107,6 @@ from the X Consortium.
 #ifndef WIN32
 #include <pwd.h>
 #endif
-#include "lncmd.h"
 #include "xpr.h"
 #include <X11/XWDFile.h>
 #include <X11/Xmu/SysUtil.h>
@@ -116,14 +138,15 @@ int debug = 0;
 #define W_PAGE 2550
 #define H_PAGE 3225
 
+#define PAPER_WIDTH 85*30 /* 8.5 inches */
+#define PAPER_LENGTH 11*300 /* 11 inches */
+
 #ifdef NOINLINE
 #define min(x,y) (((x)<(y))?(x):(y))
 #endif /* NOINLINE */
 
 #define F_PORTRAIT 1
 #define F_LANDSCAPE 2
-#define F_DUMP 4
-#define F_NOSIXOPT 8
 #define F_APPEND 16
 #define F_NOFF 32
 #define F_REPORT 64
@@ -158,6 +181,51 @@ static GrayRec gray2x2 = {sizeof(grayscale2x2)/sizeof(long), 2, 2, grayscale2x2}
 static GrayRec gray3x3 = {sizeof(grayscale3x3)/sizeof(long), 3, 3, grayscale3x3};
 static GrayRec gray4x4 = {sizeof(grayscale4x4)/sizeof(long), 4, 4, grayscale4x4};
 
+typedef struct {
+	char *name;
+	double width;
+	double length;
+} PaperRec;
+
+PaperRec paper_size[] = {
+	{"Letter", 8.5,	 11.0 },
+	{"Legal",  8.5,	 14.0 },
+	{"A0",	  33.11, 46.81},
+	{"A1",	  23.39, 33.11},
+	{"A2",	  16.5,	 23.39},
+	{"A3",	  11.7,	 16.5 },
+	{"A4",	   8.27, 11.7 },
+	{"A5",	   5.83,  8.27},
+	{"A6",	   4.13,  5.83},
+	{"A7",	   2.91,  4.13},
+	{"A8",	   2.05,  2.91},
+	{"A9",	   1.46,  2.05},
+	{"A10",	   1.02,  1.46},
+	{"B0",	  39.37, 55.67},
+	{"B1",	  27.83, 39.37},
+	{"B2",	  19.69, 27.83},
+	{"B3",	  13.90, 19.69},
+	{"B4",	   9.84, 13.90},
+	{"B5",	   6.93,  9.84},
+	{"B6",	   4.92,  6.93},
+	{"B7",	   3.46,  4.92},
+	{"B8",	   2.44,  3.46},
+	{"B9",	   1.73,  2.44},
+	{"B10",	   1.22,  1.73},
+	{"C0",	  36.10, 51.06},
+	{"C1",	  25.51, 36.10},
+	{"C2",	  18.03, 25.51},
+	{"C3",	  12.76, 18.03},
+	{"C4",	   9.02, 12.76},
+	{"C5",	   6.38,  9.02},
+	{"C6",	   4.49,  6.38},
+	{"C7",	   3.19,  4.49},
+	{"C8",	   2.24,  3.19},
+	{"C9",	   1.57,  2.24},
+	{"C10",	   1.10,  1.57}
+};
+
+
 /* mapping tables to map a byte in to the hex representation of its
  * bit-reversal
  */
@@ -195,7 +263,9 @@ void parse_args(
   int *density,
   unsigned int *cutoff,
   float *gamma,
-  int *render);
+  int *render,
+  int *paper_width,
+  int *paper_length);
 static
 void setup_layout(
   enum device device,
@@ -207,7 +277,9 @@ void setup_layout(
   char *header,
   char *trailer,
   int *scale,
-  enum orientation *orientation);
+  enum orientation *orientation,
+  int wmax,
+  int hmax);
 static
 char *convert_data(
     XWDFileHeader *win,
@@ -216,35 +288,6 @@ char *convert_data(
     GrayPtr gray,
     XColor *colors,
     int flags);
-static
-void dump_sixmap(
-  register unsigned char (*sixmap)[],
-  int iw,
-  int ih);
-static
-void build_sixmap(
-  int ih,
-  int iw,
-  unsigned char (*sixmap)[],
-  int hpad,
-  XWDFileHeader *win,
-  const char *data);
-static
-void ln03_setup(
-  int iw,
-  int ih,
-  enum orientation orientation,
-  int scale,
-  int left,
-  int top,
-  int *left_margin,
-  int *top_margin,
-  int flags,
-  const char *header,
-  const char *trailer);
-static void ln03_finish(void);
-static void la100_setup(int iw, int ih, int scale);
-static void la100_finish(void);
 static void dump_prolog(int flags);
 static int points(int n);
 static char *escape(const char *s);
@@ -259,23 +302,10 @@ void ps_setup(
   int flags,
   const char *header,
   const char *trailer,
-  const char *name);
+  const char *name,
+  int paper_width,
+  int paper_length);
 static void ps_finish(void);
-static
-void ln03_output_sixels(
-  unsigned char (*sixmap)[],
-  int iw,
-  int ih,
-  int nosixopt,
-  int split,
-  int scale,
-  int top_margin,
-  int left_margin);
-static void la100_output_sixels(
-  unsigned char (*sixmap)[],
-  int iw,
-  int ih,
-  int nosixopt);
 static void ps_output_bits(
   int iw,
   int ih,
@@ -303,16 +333,12 @@ int main(int argc, char **argv)
 {
     unsigned long swaptest = 1;
     XWDFileHeader win;
-    register unsigned char (*sixmap)[];
     register int i;
     register int iw;
     register int ih;
-    register int sixel_count;
     char *w_name;
     int scale, width, height, flags, split;
     int left, top;
-    int top_margin, left_margin;
-    int hpad;
     char *header, *trailer;
     int plane;
     int density, render;
@@ -324,6 +350,10 @@ int main(int argc, char **argv)
     enum orientation orientation;
     enum device device;
     XColor *colors = (XColor *)NULL;
+    int paper_width  = PAPER_WIDTH;
+    int paper_length = PAPER_LENGTH;
+    int w_max        = W_MAX;
+    int h_max        = W_MAX;
 
     if (!(progname = argv[0]))
       progname = "xpr";
@@ -332,7 +362,12 @@ int main(int argc, char **argv)
 #endif
     parse_args (argc, argv, &scale, &width, &height, &left, &top, &device,
 		&flags, &split, &header, &trailer, &plane, &gray,
-		&density, &cutoff, &gamma, &render);
+		&density, &cutoff, &gamma, &render, &paper_width, &paper_length);
+
+    if (paper_width != PAPER_WIDTH)
+       w_max = paper_width - 150;
+    if (paper_length != PAPER_LENGTH)
+       h_max = paper_length - 150;
 
     if (device == PP) {
 	x2pmp(stdin, stdout, scale,
@@ -356,7 +391,10 @@ int main(int argc, char **argv)
 	      (flags & F_SLIDE),
 	      device, cutoff, gamma, render);
 	exit(0);
+    } else if (device != PS) {
+	fprintf(stderr, "xpr: device not supported\n");
     }
+    /* everything past here is for device == PS */
 
     /* read in window header */
     fullread(0, (char *)&win, sizeof win);
@@ -377,7 +415,7 @@ int main(int argc, char **argv)
 
     if(win.ncolors) {
 	XWDColor xwdcolor;
-	colors = (XColor *)malloc((unsigned) (win.ncolors * sizeof(XColor)));
+	colors = malloc((unsigned) (win.ncolors * sizeof(XColor)));
 	for (i = 0; i < win.ncolors; i++) {
 	    fullread(0, (char*)&xwdcolor, (int) sizeof xwdcolor);
 	    colors[i].pixel = xwdcolor.pixel;
@@ -418,44 +456,16 @@ int main(int argc, char **argv)
 
     /* calculate orientation and scale */
     setup_layout(device, (int) win.pixmap_width, (int) win.pixmap_height,
-		 flags, width, height, header, trailer, &scale, &orientation);
+		 flags, width, height, header, trailer, &scale, &orientation,
+		 w_max, h_max);
 
-    if (device == PS) {
-	iw = win.pixmap_width;
-	ih = win.pixmap_height;
-	sixmap = NULL;
-    } else {
-	/* calculate w and h cell count */
-	iw = win.pixmap_width;
-	ih = (win.pixmap_height + 5) / 6;
-	hpad = (ih * 6) - win.pixmap_height;
+    iw = win.pixmap_width;
+    ih = win.pixmap_height;
 
-	/* build pixcells from input file */
-	sixel_count = iw * ih;
-	sixmap = (unsigned char (*)[])malloc((unsigned)sixel_count);
-	build_sixmap(iw, ih, sixmap, hpad, &win, data);
-    }
-
-    /* output commands and sixel graphics */
-    if (device == LN03) {
-/*	ln03_grind_fonts(sixmap, iw, ih, scale, &pixmap); */
-	ln03_setup(iw, ih, orientation, scale, left, top,
-		   &left_margin, &top_margin, flags, header, trailer);
-	ln03_output_sixels(sixmap, iw, ih, (flags & F_NOSIXOPT), split,
-			   scale, top_margin, left_margin);
-	ln03_finish();
-    } else if (device == LA100) {
-	la100_setup(iw, ih, scale);
-	la100_output_sixels(sixmap, iw, ih, (flags & F_NOSIXOPT));
-	la100_finish();
-    } else if (device == PS) {
-	ps_setup(iw, ih, orientation, scale, left, top,
-		   flags, header, trailer, w_name);
-	ps_output_bits(iw, ih, flags, orientation, &win, data);
-	ps_finish();
-    } else {
-	fprintf(stderr, "xpr: device not supported\n");
-    }
+    ps_setup(iw, ih, orientation, scale, left, top,
+             flags, header, trailer, w_name, paper_width, paper_length);
+    ps_output_bits(iw, ih, flags, orientation, &win, data);
+    ps_finish();
 
     /* print some statistics */
     if (flags & F_REPORT) {
@@ -465,8 +475,6 @@ int main(int argc, char **argv)
 	fprintf(stderr, "Orientation: %s, Scale: %d\n",
 		(orientation==PORTRAIT) ? "Portrait" : "Landscape", scale);
     }
-    if (((device == LN03) || (device == LA100)) && (flags & F_DUMP))
-	dump_sixmap(sixmap, iw, ih);
     exit(EXIT_SUCCESS);
 }
 
@@ -498,7 +506,7 @@ usage(void)
     fprintf(stderr, "usage: %s [options] [file]\n%s", progname,
 	    "    -append <file>  -noff  -output <file>\n"
 	    "    -compact\n"
-	    "    -device {ln03 | la100 | ps | lw | pp | ljet | pjet | pjetxl}\n"
+	    "    -device {ps | lw | pp | ljet | pjet | pjetxl}\n"
 	    "    -dump\n"
 	    "    -gamma <correction>\n"
 	    "    -gray {2 | 3 | 4}\n"
@@ -507,7 +515,8 @@ usage(void)
 	    "    -landscape  -portrait\n"
 	    "    -left <inches>  -top <inches>\n"
 	    "    -noposition\n"
-	    "    -nosixopt\n"
+	    "    -papertype {letter | legal | A0-A10 | B0-B10 | C0-C10}\n"
+	    "	 -pheight <inches>[in | cm | mm] -pwidth <inches>[in | cm | mm]\n"
 	    "    -plane <n>\n"
 	    "    -psfig\n"
 	    "    -render <type>\n"
@@ -540,11 +549,12 @@ void parse_args(
   int *density,
   unsigned int *cutoff,
   float *gamma,
-  int *render)
+  int *render,
+  int *paper_width,
+  int *paper_length)
 {
     register char *output_filename;
     register int f;
-    register int pos;
 
     output_filename = NULL;
     *device = PS;	/* default */
@@ -581,8 +591,6 @@ void parse_args(
 	    argc--; argv++;
 	    if (argc == 0) missing_arg(arg);
 	    *cutoff = min((atof(*argv) / 100.0 * 0xFFFF), 0xFFFF);
-	} else if (!strcmp(*argv, "-dump")) {
-	    *flags |= F_DUMP;
 	} else if (!strcmp(*argv, "-density")) {
 	    argc--; argv++;
 	    if (argc == 0) missing_arg(arg);
@@ -590,11 +598,7 @@ void parse_args(
 	} else if (!strcmp(*argv, "-device")) {
 	    argc--; argv++;
 	    if (argc == 0) missing_arg(arg);
-	    if (!strcmp(*argv, "ln03")) {
-		*device = LN03;
-	    } else if (!strcmp(*argv, "la100")) {
-		*device = LA100;
-	    } else if (!strcmp(*argv, "ps")) {
+	    if (!strcmp(*argv, "ps")) {
 		*device = PS;
 	    } else if (!strcmp(*argv, "lw")) {
 		*device = PS;
@@ -644,8 +648,6 @@ void parse_args(
 	    argc--; argv++;
 	    if (argc == 0) missing_arg(arg);
 	    *left = (int)(300.0 * atof(*argv));
-	} else if (!strcmp(*argv, "-nosixopt")) {
-	    *flags |= F_NOSIXOPT;
 	} else if (!strcmp(*argv, "-noff")) {
 	    *flags |= F_NOFF;
 	} else if (!strcmp(*argv, "-noposition")) {
@@ -656,12 +658,48 @@ void parse_args(
 	    output_filename = *argv;
 	} else if (!strcmp(*argv, "-portrait")) {
 	    *flags |= F_PORTRAIT;
+	} else if (!strcmp(*argv, "-papertype")) {
+	    int found = 0;
+	    int i;
+
+	    argc--; argv++;
+	    if (argc == 0) missing_arg(arg);
+	    for (i = 0; i < sizeof (paper_size) / sizeof (PaperRec); i++)
+	    {
+		if (!strcasecmp(paper_size[i].name, *argv))
+		{
+		    found = 1;
+		    *paper_width  = (int)(300.0 * paper_size[i].width);
+		    *paper_length = (int)(300.0 * paper_size[i].length);
+		}
+	    }
+	    if (!found) usage();
 	} else if (!strcmp(*argv, "-plane")) {
 	    argc--; argv++;
 	    if (argc == 0) missing_arg(arg);
 	    *plane = atoi(*argv);
 	} else if (!strcmp(*argv, "-psfig")) {
 	    *flags |= F_NPOSITION;
+	} else if (!strcmp(*argv, "-pwidth")) {
+	    double mult = 1.0;
+
+	    argc--; argv++;
+	    if (argc == 0) missing_arg(arg);
+	    if (strstr(*argv, "cm"))
+		mult = 2.54;
+	    else if ( strstr ( *argv, "mm" ))
+		mult = 25.4;
+	    *paper_width = (int)(300.0 * atof(*argv) / mult);
+	} else if (!strcmp(*argv, "-pheight")) {
+	    double mult = 1.0;
+
+	    argc--; argv++;
+	    if (argc == 0) missing_arg(arg);
+	    if ( strstr ( *argv, "cm" ))
+		mult = 2.54;
+	    else if ( strstr ( *argv, "mm" ))
+		mult = 25.4;
+	    *paper_length = (int)(300.0 * atof(*argv) / mult);
 	} else if (!strcmp(*argv, "-rv")) {
 	    *flags |= F_INVERT;
 	} else if (!strcmp(*argv, "-render")) {
@@ -700,7 +738,7 @@ void parse_args(
     }
 
     if (infilename) {
-	f = open(infilename, O_RDONLY|O_BINARY, 0);
+	f = open(infilename, O_RDONLY|O_BINARY);
 	if (f < 0) {
 	    fprintf(stderr, "xpr: error opening \"%s\" for input\n",
 		    infilename);
@@ -725,12 +763,12 @@ void parse_args(
 	    exit(1);
 	}
 	if (*flags & F_APPEND) {
-	    pos = lseek(f, 0, 2);          /* get eof position */
+	    off_t pos = lseek(f, 0, SEEK_END);	/* get eof position */
 	    if ((*flags & F_NOFF) &&
 		!(*device == LJET || *device == PJET || *device == PJETXL))
 		pos -= 3; /* set position before trailing */
 	    		  /*     formfeed and reset */
-	    lseek(f, pos, 0);              /* set pointer */
+	    lseek(f, pos, SEEK_SET);		/* set pointer */
 	}
 	dup2(f, 1);
 	close(f);
@@ -748,7 +786,9 @@ void setup_layout(
   char *header,
   char *trailer,
   int *scale,
-  enum orientation *orientation)
+  enum orientation *orientation,
+  int wmax,
+  int hmax)
 {
     register int w_scale;
     register int h_scale;
@@ -760,26 +800,26 @@ void setup_layout(
     if (trailer != NULL) win_height += 75;
 
     /* check maximum width and height; set orientation and scale*/
-    if (device == LN03 || device == PS) {
+    if (device == PS) {
 	if ((win_width < win_height || (flags & F_PORTRAIT)) &&
 	    !(flags & F_LANDSCAPE)) {
 	    *orientation = PORTRAIT;
-	    w_max = (width > 0)? width : W_MAX;
-	    h_max = (height > 0)? height : H_MAX;
+	    w_max = (width > 0)? width : wmax;
+	    h_max = (height > 0)? height : hmax;
 	    w_scale = w_max / win_width;
 	    h_scale = h_max / win_height;
 	    *scale = min(w_scale, h_scale);
 	} else {
 	    *orientation = LANDSCAPE;
-	    w_max = (width > 0)? width : H_MAX;
-	    h_max = (height > 0)? height : W_MAX;
+	    w_max = (width > 0)? width : hmax;
+	    h_max = (height > 0)? height : wmax;
 	    w_scale = w_max / win_width;
 	    h_scale = h_max / win_height;
 	    *scale = min(w_scale, h_scale);
 	}
-    } else {			/* device == LA100 */
+    } else {
 	*orientation = PORTRAIT;
-	*scale = W_MAX / win_width;
+	*scale = wmax / win_width;
     }
     if (*scale == 0) *scale = 1;
     if (*scale > 6) *scale = 6;
@@ -980,214 +1020,6 @@ char *convert_data(
     return (out_image->data);
 }
 
-static
-void dump_sixmap(
-  register unsigned char (*sixmap)[],
-  int iw,
-  int ih)
-{
-    register int i, j;
-    register unsigned char *c;
-
-    c = (unsigned char *)sixmap;
-    fprintf(stderr, "Sixmap:\n");
-    for (i = 0; i < ih; i++) {
-	for (j = 0; j < iw; j++) {
-	    fprintf(stderr, "%02X ", *c++);
-	}
-	fprintf(stderr, "\n\n");
-    }
-}
-
-static
-void build_sixmap(
-  int ih,
-  int iw,
-  unsigned char (*sixmap)[],
-  int hpad,
-  XWDFileHeader *win,
-  const char *data)
-{
-    int iwb = win->bytes_per_line;
-    unsigned char *line[6];
-    register unsigned char *c;
-    register int i, j;
-#ifdef NOINLINE
-    register int w;
-#endif
-    register int sixel;
-    unsigned char *buffer = (unsigned char *)data;
-
-    c = (unsigned char *)sixmap;
-
-
-    while (--ih >= 0) {
-        for (i = 0; i <= 5; i++) {
-	    line[i] = buffer;
-	    buffer += iwb;
-        }
-	if ((ih == 0) && (hpad > 0)) {
-	    unsigned char *ffbuf;
-
-	    ffbuf = (unsigned char *)malloc((unsigned)iwb);
-	    for (j = 0; j < iwb; j++)
-		ffbuf[j] = 0xFF;
-	    for (; --hpad >= 0; i--)
-		line[i] = ffbuf;
-	}
-
-#ifndef NOINLINE
-	for (i = 0; i < iw; i++) {
-	    sixel =  extzv(line[0], i, 1);
-	    sixel |= extzv(line[1], i, 1) << 1;
-	    sixel |= extzv(line[2], i, 1) << 2;
-	    sixel |= extzv(line[3], i, 1) << 3;
-	    sixel |= extzv(line[4], i, 1) << 4;
-	    sixel |= extzv(line[5], i, 1) << 5;
-	    *c++ = sixel;
-	}
-#else
-	for (i = 0, w = iw; w > 0; i++) {
-	    for (j = 0; j <= 7; j++) {
-		sixel =  ((line[0][i] >> j) & 1);
-		sixel |= ((line[1][i] >> j) & 1) << 1;
-		sixel |= ((line[2][i] >> j) & 1) << 2;
-		sixel |= ((line[3][i] >> j) & 1) << 3;
-		sixel |= ((line[4][i] >> j) & 1) << 4;
-		sixel |= ((line[5][i] >> j) & 1) << 5;
-		*c++ = sixel;
-		if (--w == 0) break;
-	    }
-	}
-#endif
-    }
-}
-
-/*
-ln03_grind_fonts(sixmap, iw, ih, scale, pixmap)
-unsigned char (*sixmap)[];
-int iw;
-int ih;
-int scale;
-struct pixmap (**pixmap)[];
-{
-}
-*/
-
-static
-void ln03_setup(
-  int iw,
-  int ih,
-  enum orientation orientation,
-  int scale,
-  int left,
-  int top,
-  int *left_margin,
-  int *top_margin,
-  int flags,
-  const char *header,
-  const char *trailer)
-{
-    register int i;
-    register int lm, tm, xm;
-    char buf[256];
-    register char *bp = buf;
-
-    if (!(flags & F_APPEND)) {
-	sprintf(bp, LN_STR); bp += 4;
-	sprintf(bp, LN_SSU, 7); bp += 5;
-	sprintf(bp, LN_PUM_SET); bp += sizeof LN_PUM_SET - 1;
-    }
-
-    if (orientation == PORTRAIT) {
-	lm = (left > 0)? left : (((W_MAX - scale * iw) / 2) + W_MARGIN);
-	tm = (top > 0)? top : (((H_MAX - scale * ih * 6) / 2) + H_MARGIN);
-	sprintf(bp, LN_PFS, "?20"); bp += 7;
-	sprintf(bp, LN_DECOPM_SET); bp += sizeof LN_DECOPM_SET - 1;
-	sprintf(bp, LN_DECSLRM, lm, W_PAGE - lm); bp += strlen(bp);
-    } else {
-	lm = (left > 0)? left : (((H_MAX - scale * iw) / 2) + H_MARGIN);
-	tm = (top > 0)? top : (((W_MAX - scale * ih * 6) / 2) + W_MARGIN);
-	sprintf(bp, LN_PFS, "?21"); bp += 7;
-	sprintf(bp, LN_DECOPM_SET); bp += sizeof LN_DECOPM_SET - 1;
-	sprintf(bp, LN_DECSLRM, lm, H_PAGE - lm); bp += strlen(bp);
-    }
-
-    if (header != NULL) {
-	sprintf(bp, LN_VPA, tm - 100); bp += strlen(bp);
-	i = strlen(header);
-	xm = (((scale * iw) - (i * 30)) / 2) + lm;
-	sprintf(bp, LN_HPA, xm); bp += strlen(bp);
-	sprintf(bp, LN_SGR, 3); bp += strlen(bp);
-	memmove(bp, header, i);
-	bp += i;
-    }
-    if (trailer != NULL) {
-	sprintf(bp, LN_VPA, tm + (scale * ih * 6) + 75); bp += strlen(bp);
-	i = strlen(trailer);
-	xm = (((scale * iw) - (i * 30)) / 2) + lm;
-	sprintf(bp, LN_HPA, xm); bp += strlen(bp);
-	sprintf(bp, LN_SGR, 3); bp += strlen(bp);
-	memmove(bp, trailer, i);
-	bp += i;
-    }
-
-    sprintf(bp, LN_HPA, lm); bp += strlen(bp);
-    sprintf(bp, LN_VPA, tm); bp += strlen(bp);
-    sprintf(bp, LN_SIXEL_GRAPHICS, 9, 0, scale); bp += strlen(bp);
-    sprintf(bp, "\"1;1"); bp += 4; /* Pixel aspect ratio */
-    write(1, buf, bp-buf);
-    *top_margin = tm;
-    *left_margin = lm;
-}
-
-static
-void ln03_finish(void)
-{
-    char buf[256];
-    register char *bp = buf;
-
-    sprintf(bp, LN_DECOPM_RESET); bp += sizeof LN_DECOPM_SET - 1;
-    sprintf(bp, LN_LNM); bp += 5;
-    sprintf(bp, LN_PUM); bp += 5;
-    sprintf(bp, LN_PFS, "?20"); bp += 7;
-    sprintf(bp, LN_SGR, 0); bp += strlen(bp);
-    sprintf(bp, LN_HPA, 1); bp += strlen(bp);
-    sprintf(bp, LN_VPA, 1); bp += strlen(bp);
-
-
-    write(1, buf, bp-buf);
-}
-
-/*ARGSUSED*/
-static
-void la100_setup(int iw, int ih, int scale)
-{
-    char buf[256];
-    register char *bp;
-    int lm, tm;
-
-    bp = buf;
-    lm = ((80 - (int)((double)iw / 6.6)) / 2) - 1;
-    if (lm < 1) lm = 1;
-    tm = ((66 - (int)((double)ih / 2)) / 2) - 1;
-    if (tm < 1) tm = 1;
-    sprintf(bp, "\033[%d;%ds", lm, 81-lm); bp += strlen(bp);
-    sprintf(bp, "\033[?7l"); bp += 5;
-    sprintf(bp, "\033[%dd", tm); bp += strlen(bp);
-    sprintf(bp, "\033[%d`", lm); bp += strlen(bp);
-    sprintf(bp, "\033P0q"); bp += 4;
-    write(1, buf, bp-buf);
-}
-
-#define LA100_RESET "\033[1;80s\033[?7h"
-
-static
-void la100_finish(void)
-{
-    write(1, LA100_RESET, sizeof LA100_RESET - 1);
-}
-
 #define COMMENTVERSION "PS-Adobe-1.0"
 
 #ifdef XPROLOG
@@ -1368,7 +1200,9 @@ void ps_setup(
   int flags,
   const char *header,
   const char *trailer,
-  const char *name)
+  const char *name,
+  int paper_width,
+  int paper_length)
 {
     char    hostname[256];
 #ifdef WIN32
@@ -1376,7 +1210,7 @@ void ps_setup(
 #else
     struct passwd  *pswd;
 #endif
-    long    clock;
+    time_t    clock;
     int lm, bm; /* left (bottom) margin */
 
     /* calculate margins */
@@ -1487,148 +1321,6 @@ void ps_finish(void)
 	while (*p) printf("%s\n",*p++);
 }
 
-static
-void ln03_output_sixels(
-  unsigned char (*sixmap)[],
-  int iw,
-  int ih,
-  int nosixopt,
-  int split,
-  int scale,
-  int top_margin,
-  int left_margin)
-{
-    unsigned char *buf;
-    register unsigned char *bp;
-    int i;
-    int j;
-    register int k;
-    register unsigned char *c;
-    register int lastc;
-    register int count;
-    char snum[6];
-    register char *snp;
-
-    bp = (unsigned char *)malloc((unsigned)(iw*ih+512));
-    buf = bp;
-    count = 0;
-    lastc = -1;
-    c = (unsigned char *)sixmap;
-    split = ih / split;		/* number of lines per page */
-
-    iw--;			/* optimization */
-    for (i = 0; i < ih; i++) {
-	for (j = 0; j <= iw; j++) {
-	    if (!nosixopt) {
-		if (*c == lastc && j < iw) {
-		    count++;
-		    c++;
-		    continue;
-		}
-		if (count >= 3) {
-		    bp--;
-		    count++;
-		    *bp++ = '!';
-		    snp = snum;
-		    while (count > 0) {
-			k = count / 10;
-			*snp++ = count - (k * 10) + '0';
-			count = k;
-		    }
-		    while (--snp >= snum) *bp++ = *snp;
-		    *bp++ = (~lastc & 0x3F) + 0x3F;
-		} else if (count > 0) {
-		    lastc = (~lastc & 0x3F) + 0x3F;
-		    do {
-			*bp++ = lastc;
-		    } while (--count > 0);
-		}
-	    }
-	    lastc = *c++;
-	    *bp++ = (~lastc & 0x3F) + 0x3F;
-	}
-	*bp++ = '-';		/* New line */
-	lastc = -1;
-	if ((i % split) == 0 && i != 0) {
-	    sprintf((char *)bp, LN_ST); bp += sizeof LN_ST - 1;
-	    *bp++ = '\f';
-	    sprintf((char *)bp, LN_VPA, top_margin + (i * 6 * scale));
-	    bp += strlen((char *)bp);
-	    sprintf((char *)bp, LN_HPA, left_margin);
-	    bp += strlen((char *)bp);
-	    sprintf((char *)bp, LN_SIXEL_GRAPHICS, 9, 0, scale);
-	    bp += strlen((char *)bp);
-	    sprintf((char *)bp, "\"1;1"); bp += 4;
-	}
-    }
-
-    sprintf((char *)bp, LN_ST); bp += sizeof LN_ST - 1;
-    write(1, (char *)buf, bp-buf);
-}
-
-/*ARGSUSED*/
-static
-void la100_output_sixels(
-  unsigned char (*sixmap)[],
-  int iw,
-  int ih,
-  int nosixopt)
-{
-    unsigned char *buf;
-    register unsigned char *bp;
-    int i;
-    register int j, k;
-    register unsigned char *c;
-    register int lastc;
-    register int count;
-    char snum[6];
-
-    bp = (unsigned char *)malloc((unsigned)(iw*ih+512));
-    buf = bp;
-    count = 0;
-    lastc = -1;
-    c = (unsigned char *)sixmap;
-
-    for (i = 0; i < ih; i++) {
-	for (j = 0; j < iw; j++) {
-	    if (*c == lastc && (j+1) < iw) {
-		count++;
-		c++;
-		continue;
-	    }
-	    if (count >= 2) {
-		bp -= 2;
-		count = 2 * (count + 1);
-		*bp++ = '!';
-		k = 0;
-		while (count > 0) {
-		    snum[k++] = (count % 10) + '0';
-		    count /= 10;
-		}
-		while (--k >= 0) *bp++ = snum[k];
-		*bp++ = (~lastc & 0x3F) + 0x3F;
-		count = 0;
-	    } else if (count > 0) {
-		lastc = (~lastc & 0x3F) + 0x3F;
-		do {
-		    *bp++ = lastc;
-		    *bp++ = lastc;
-		} while (--count > 0);
-	    }
-	    lastc = (~*c & 0x3F) + 0x3F;
-	    *bp++ = lastc;
-	    *bp++ = lastc;
-	    lastc = *c++;
-	}
-	*bp++ = '-';		/* New line */
-	lastc = -1;
-    }
-
-    sprintf((char *)bp, LN_ST); bp += sizeof LN_ST - 1;
-    *bp++ = '\f';
-    write(1, (char *)buf, bp-buf);
-}
-
 #define LINELEN 72 /* number of CHARS (bytes*2) per line of bitmap output */
 
 static
@@ -1668,16 +1360,15 @@ void ps_output_bits(
 	 * Owidth and Oheight are rounded up to a multiple of 32 bits,
 	 * to avoid special cases at the boundaries
 	 */
-	obuf = malloc((unsigned)(owidth*oheight));
+	obuf = calloc(owidth, oheight);
 	if (obuf==NULL) {
 	    fprintf(stderr,"xpr: cannot allocate %d bytes\n",owidth*oheight);
 	    exit(1);
 	}
-	bzero(obuf,owidth*oheight);
 
-	ibuf = (unsigned char *)malloc((unsigned)(iwb + 3));
+	ibuf = malloc((unsigned)(iwb + 3));
 	for (i=0;i<ih;i++) {
-	    memmove((char *)ibuf, (char *)buffer, iwb);
+	    memcpy(ibuf, buffer, iwb);
 	    buffer += iwb;
 	    if (!(*(char *) &swaptest))
 		_swaplong((char *)ibuf,(long)iwb);
